package mongoauth;

import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.TimeUnit;

import com.mongodb.DB;
import com.mongodb.Mongo;

/**
 * The purpose of this program is to demonstrate that
 * <code>db.authenticate(...)</code> does not perform consistently when used
 * from multiple threads. The mongod log also shows a message '<i>auth: bad
 * nonce received or getnonce not called. could be a driver bug or a security
 * attack</i>' whenever authentication fails.
 */
public class MongoAuthentication {

	// FIXME provide correct values for your mongodb
	private static final String HOST = "localhost";
	private static final int PORT = 27017;
	private static final String DB_NAME = ".....";
	private static final String USERNAME = ".....";
	private static final char[] PASSWORD = ".....".toCharArray();

	public static void main(String[] args) throws Exception {

		final ExecutorService executors = Executors.newFixedThreadPool(50);
		final DB db = new Mongo(HOST, PORT).getDB(DB_NAME);

		for (int i = 0; i < 1000; i++) {
			executors.submit(new Runnable() {

				@Override
				public void run() {
					if (!db.isAuthenticated()) {
						boolean authenticated = db.authenticate(USERNAME, PASSWORD);

						if (authenticated) {
							System.out.println("Authentication successful");
						} else {
							System.err.println("Authentication failed");
							executors.shutdownNow();
						}
					}
				}
			});
		}

		if (!executors.isShutdown()) {
			executors.shutdown();
		}

		executors.awaitTermination(5, TimeUnit.SECONDS);
	}
}